﻿Type=Class
Version=2.02
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private pnlContent As Panel
	Private pnlSidebar As Panel
	Private pnlGesture As Panel
	Private sbParent As Panel
	Private sbPosition As Byte
	Private sbAnimType As Byte
	Private Anim1, Anim2, Anim3 As AnimationPlus
	Private sbOpenDuration As Int
	Private sbCloseDuration As Int
	Private sbHandle As View
	Private sbIsVisible As Boolean
	Private sbStartX, sbStartY As Int
	Private sbVelocityTracker As Object
	Private sbMinVelocity As Float: sbMinVelocity = 0.5
End Sub

'Initializes the content panel and its sidebar
'<B>Parent</B> = activity or panel holding the content panel and its sidebar
'<B>SidebarSize</B> = width or height of the sidebar
'<B>SidebarPosition</B>:<I>
' 0 = left
' 1 = right
' 2 = top
' 3 = bottom</I>
'<B>AnimType</B>:<I>
' 0 = the content panel is pushed to the side and reveals the sidebar
' 1 = the content panel and the sidebar move together to the side
' 2 = the sidebar slides above the content panel</I>
'<B>Open</B>/<B>CloseDuration</B> = duration of the opening/closing animation
'<B>Module</B> = Me
Public Sub Initialize(Parent As Panel, SidebarSize As Int, SidebarPosition As Byte, AnimType As Byte, _
                      OpenDuration As Int, CloseDuration As Int, Module As Object)
	sbParent = Parent
	pnlContent.Initialize("")
	Parent.AddView(pnlContent, 0, 0, getParentWidth, getParentHeight)

	pnlSidebar.Initialize("")
	Select SidebarPosition
		Case 0 'LEFT
			If AnimType = 0 Then
				Parent.AddView(pnlSidebar, 0, 0, SidebarSize, getParentHeight)
			Else
				Parent.AddView(pnlSidebar, -SidebarSize, 0, SidebarSize, getParentHeight)
			End If
		Case 1 'RIGHT
			If AnimType = 0 Then
				Parent.AddView(pnlSidebar, getParentWidth - SidebarSize, 0, SidebarSize, getParentHeight)
			Else
				Parent.AddView(pnlSidebar, getParentWidth, 0, SidebarSize, getParentHeight)
			End If
		Case 2 'TOP
			If AnimType = 0 Then
				Parent.AddView(pnlSidebar, 0, 0, getParentWidth, SidebarSize)
			Else
				Parent.AddView(pnlSidebar, 0, -SidebarSize, getParentWidth, SidebarSize)
			End If
		Case 3 'BOTTOM
			If AnimType = 0 Then
				Parent.AddView(pnlSidebar, 0, getParentHeight - SidebarSize, getParentWidth, SidebarSize)
			Else
				Parent.AddView(pnlSidebar, 0, getParentHeight, getParentWidth, SidebarSize)
			End If
	End Select
	If AnimType = 0 Then pnlSidebar.SendToBack

	sbPosition = SidebarPosition
	sbAnimType = AnimType
	sbOpenDuration = OpenDuration
	sbCloseDuration = CloseDuration
	sbIsVisible = False

'	Dim r As Reflector
'	r.Target = r.RunStaticMethod("android.view.ViewConfiguration", "get", Array As Object(r.GetContext), Array As String("android.content.Context"))
'	sbTouchSlop = r.RunMethod("getScaledTouchSlop") / 2
	sbVelocityTracker = Null
End Sub

'Gets a drawable from the Android system resources
Public Sub LoadDrawable(Name As String) As Object
	Dim r As Reflector
	r.Target = r.GetContext
	r.Target = r.RunMethod("getResources")
	r.Target = r.RunMethod("getSystem")
	Dim ID_Drawable As Int
	ID_Drawable = r.RunMethod4("getIdentifier", Array As Object(Name, "drawable", "android"), _
	                                            Array As String("java.lang.String", "java.lang.String", "java.lang.String"))
	r.Target = r.GetContext
	r.Target = r.RunMethod("getResources")
	Return r.RunMethod2("getDrawable", ID_Drawable, "java.lang.int")
End Sub

Public Sub Sidebar As Panel
	Return pnlSidebar
End Sub

Public Sub ContentPanel As Panel
	Return pnlContent
End Sub

#Region Parent Width/Height
'Gets the real width of the parent
'In some containers like TabHosts, the width property returns -1, so this function uses a different method to get width
Private Sub getParentWidth As Int
	If sbParent.Width < 0 Then
		Dim r As Reflector, RealWidth As Int
		r.Target = sbParent
		RealWidth = r.RunMethod("getWidth")
		If RealWidth = 0 Then
			DoEvents
			RealWidth = r.RunMethod("getWidth")
		End If
		Return RealWidth
	Else
		Return sbParent.Width
	End If
End Sub

'Gets the real height of the parent
'In some containers like TabHosts, the height property returns -1, so this function uses a different method to get height
Private Sub getParentHeight As Int
	If sbParent.Height < 0 Then
		Dim r As Reflector, RealHeight As Int
		r.Target = sbParent
		RealHeight = r.RunMethod("getHeight")
		If RealHeight = 0 Then
			DoEvents
			RealHeight = r.RunMethod("getHeight")
		End If
		Return RealHeight
	Else
		Return sbParent.Height
	End If
End Sub
#End Region

Private Sub AnimateSidebar(Close As Boolean, AnimDuration As Int)
	If Anim1.IsInitialized Then Anim1.Stop(pnlSidebar)
	If Anim2.IsInitialized Then Anim2.Stop(pnlContent)
	If Anim3.IsInitialized AND sbHandle.IsInitialized Then Anim3.Stop(sbHandle)
	If pnlGesture.IsInitialized Then pnlGesture.Enabled = False

	Dim EventName As String
	If Close Then
		EventName = "AnimClose"
	Else
		EventName = "AnimOpen"
	End If

	Dim MoveQty As Int
	Select sbPosition
		Case 0 'LEFT
			If sbAnimType > 0 Then
				If Close Then
					MoveQty = - pnlSidebar.Left - pnlSidebar.Width
				Else
					MoveQty = pnlSidebar.Width
				End If
				Anim1.InitializeTranslate(EventName, 0, 0, MoveQty, 0)
			End If
			If sbAnimType < 2 Then
				If Close Then
					MoveQty = -pnlContent.Left
				Else
					MoveQty = pnlSidebar.Width
				End If
				Anim2.InitializeTranslate(EventName, 0, 0, MoveQty, 0)
			End If
			If sbHandle.IsInitialized Then Anim3.InitializeTranslate("", 0, 0, MoveQty, 0)
		Case 1 'RIGHT
			If sbAnimType > 0 Then
				If Close Then
					MoveQty = -pnlSidebar.Left + pnlContent.Width
				Else
					MoveQty = -pnlSidebar.Width
				End If
				Anim1.InitializeTranslate(EventName, 0, 0, MoveQty, 0)
			End If
			If sbAnimType < 2 Then
				If Close Then
					MoveQty = -pnlContent.Left
				Else
					MoveQty = -pnlSidebar.Width
				End If
				Anim2.InitializeTranslate(EventName, 0, 0, MoveQty, 0)
			End If
			If sbHandle.IsInitialized Then Anim3.InitializeTranslate("", 0, 0, MoveQty, 0)
		Case 2 'TOP
			If sbAnimType > 0 Then
				If Close Then
					MoveQty = -pnlSidebar.Top - pnlSidebar.Height
				Else
					MoveQty = pnlSidebar.Height
				End If
				Anim1.InitializeTranslate(EventName, 0, 0, 0, MoveQty)
			End If
			If sbAnimType < 2 Then
				If Close Then
					MoveQty = -pnlContent.Top
				Else
					MoveQty = pnlSidebar.Height
				End If
				Anim2.InitializeTranslate(EventName, 0, 0, 0, MoveQty)
			End If
			If sbHandle.IsInitialized Then Anim3.InitializeTranslate("", 0, 0, 0, MoveQty)
		Case 3 'BOTTOM
			If sbAnimType > 0 Then
				If Close Then
					MoveQty = -pnlSidebar.Top + pnlContent.Height
				Else
					MoveQty = -pnlSidebar.Height
				End If
				Anim1.InitializeTranslate(EventName, 0, 0, 0, MoveQty)
			End If
			If sbAnimType < 2 Then
				If Close Then
					MoveQty = -pnlContent.Top
				Else
					MoveQty = -pnlSidebar.Height
				End If
				Anim2.InitializeTranslate(EventName, 0, 0, 0, MoveQty)
			End If
			If sbHandle.IsInitialized Then Anim3.InitializeTranslate("", 0, 0, 0, MoveQty)
	End Select

	Dim Interpolator As Int
	Interpolator = Anim1.INTERPOLATOR_ACCELERATE_DECELERATE
	If sbAnimType > 0 Then
		Anim1.SetInterpolator(Interpolator)
		Anim1.Duration = AnimDuration
		Anim1.RepeatCount = 0
	End If
	If sbHandle.IsInitialized Then
		Anim3.SetInterpolator(Interpolator)
		Anim3.Duration = AnimDuration
		Anim3.RepeatCount = 0
	End If
	If sbAnimType < 2 Then
		Anim2.SetInterpolator(Interpolator)
		Anim2.Duration = AnimDuration
		Anim2.RepeatCount = 0
		Anim2.Start(pnlContent)
	End If
	If sbAnimType > 0 Then Anim1.Start(pnlSidebar)
	If sbHandle.IsInitialized Then Anim3.Start(sbHandle)

	sbIsVisible = Not(Close)
End Sub

Private Sub AnimOpen_AnimationEnd
	Select sbPosition
		Case 0 'LEFT
			If sbAnimType > 0 Then pnlSidebar.Left = 0
			If sbAnimType < 2 Then pnlContent.Left = pnlSidebar.Width
			If sbHandle.IsInitialized Then sbHandle.Left = pnlSidebar.Left + pnlSidebar.Width
			If pnlGesture.IsInitialized Then pnlGesture.Left = pnlSidebar.Left + pnlSidebar.Width - (pnlGesture.Width / 2)
		Case 1 'RIGHT
			If sbAnimType > 0 Then pnlSidebar.Left = pnlContent.Width - pnlSidebar.Width
			If sbAnimType < 2 Then pnlContent.Left = -pnlSidebar.Width
			If sbHandle.IsInitialized Then sbHandle.Left = pnlSidebar.Left - sbHandle.Width
			If pnlGesture.IsInitialized Then pnlGesture.Left = pnlSidebar.Left - (pnlGesture.Width / 2)
		Case 2 'TOP
			If sbAnimType > 0 Then pnlSidebar.Top = 0
			If sbAnimType < 2 Then pnlContent.Top = pnlSidebar.Height
			If sbHandle.IsInitialized Then sbHandle.Top = pnlSidebar.Top + pnlSidebar.Height
			If pnlGesture.IsInitialized Then pnlGesture.Top = pnlSidebar.Top + pnlSidebar.Height - (pnlGesture.Height / 2)
		Case 3 'BOTTOM
			If sbAnimType > 0 Then pnlSidebar.Top = pnlContent.Height - pnlSidebar.Height
			If sbAnimType < 2 Then pnlContent.Top = -pnlSidebar.Height
			If sbHandle.IsInitialized Then sbHandle.Top = pnlSidebar.Top - sbHandle.Height
			If pnlGesture.IsInitialized Then pnlGesture.Top = pnlSidebar.Top - (pnlGesture.Height / 2)
	End Select
	If pnlGesture.IsInitialized Then pnlGesture.Enabled = True
End Sub

Private Sub AnimClose_AnimationEnd
	Select sbPosition
		Case 0 'LEFT
			If sbAnimType > 0 Then pnlSidebar.Left = -pnlSidebar.Width
			If sbAnimType < 2 Then pnlContent.Left = 0
			If sbHandle.IsInitialized Then sbHandle.Left = pnlContent.Left
			If pnlGesture.IsInitialized Then pnlGesture.Left = pnlContent.Left - (pnlGesture.Width / 2)
		Case 1 'RIGHT
			If sbAnimType > 0 Then pnlSidebar.Left = pnlContent.Width
			If sbAnimType < 2 Then pnlContent.Left = 0
			If sbHandle.IsInitialized Then sbHandle.Left = pnlContent.Width - sbHandle.Width
			If pnlGesture.IsInitialized Then pnlGesture.Left = pnlContent.Left + pnlContent.Width - (pnlGesture.Width / 2)
		Case 2 'TOP
			If sbAnimType > 0 Then pnlSidebar.Top = -pnlSidebar.Height
			If sbAnimType < 2 Then pnlContent.Top = 0
			If sbHandle.IsInitialized Then sbHandle.Top = pnlContent.Top
			If pnlGesture.IsInitialized Then pnlGesture.Top = pnlContent.Top - (pnlGesture.Height / 2)
		Case 3 'BOTTOM
			If sbAnimType > 0 Then pnlSidebar.Top = pnlContent.Height
			If sbAnimType < 2 Then pnlContent.Top = 0
			If sbHandle.IsInitialized Then sbHandle.Top = pnlContent.Height - sbHandle.Height
			If pnlGesture.IsInitialized Then pnlGesture.Top = pnlContent.Top + pnlContent.Height - (pnlGesture.Height / 2)
	End Select
	If pnlGesture.IsInitialized Then pnlGesture.Enabled = True
End Sub

'Opens the sidebar
Public Sub OpenSidebar
	AnimateSidebar(False, sbOpenDuration)
End Sub

'Closes the sidebar
Public Sub CloseSidebar
	AnimateSidebar(True, sbCloseDuration)
End Sub

'Is the sidebar currently visible ?
Public Sub IsSidebarVisible As Boolean
	Return sbIsVisible
End Sub

'Sets the button used to open or close the sidebar
'The button can be a Panel, an ImageView, a Button...
'It must be initialized and on screen.
Public Sub SetOpenCloseButton(Btn As View)
	If Btn = Null Then Return
	Dim r As Reflector
	r.Target = Btn
	r.SetOnClickListener("Btn_Click")
End Sub

Private Sub Btn_Click(ViewTag As Object)
	If IsSidebarVisible Then
		CloseSidebar
	Else If Not(IsSidebarVisible) Then
		OpenSidebar
	End If
End Sub

'Adds the handle used to open or close the sidebar
'The handle can be a Panel, an ImageView, a Button...
'It must be initialized but not yet on screen.
'<B>Position</B> = top position if sidebar is on the left or the right, left position otherwise
Public Sub AddOpenCloseHandle(Hdl As View, Position As Int, Width As Int, Height As Int)
	If Hdl = Null Then Return
	sbHandle = Hdl
	Select sbPosition
		Case 0 'LEFT
			sbParent.AddView(Hdl, pnlContent.Left, Position, Width, Height)
		Case 1 'RIGHT
			sbParent.AddView(Hdl, pnlContent.Left + pnlContent.Width - Width, Position, Width, Height)
		Case 2 'TOP
			sbParent.AddView(Hdl, Position, pnlContent.Top, Width, Height)
		Case 3 'BOTTOM
			sbParent.AddView(Hdl, Position, pnlContent.Top + pnlContent.Height - Height, Width, Height)
	End Select
	Dim r As Reflector
	r.Target = Hdl
	r.SetOnTouchListener("Hdl_Touch")
End Sub

Private Sub Hdl_Touch(ViewTag As Object, Action As Int, X As Float, Y As Float, MotionEvent As Object) As Boolean
	If Action = 0 Then
		sbStartX = X
		sbStartY = Y
	Else If Action = 2 Then
		Dim OldPos As Int
		Select sbPosition
			Case 0 'LEFT
				If sbAnimType = 2 Then
					OldPos = pnlSidebar.Left
					pnlSidebar.Left = Max(-pnlSidebar.Width, Min(pnlSidebar.Left + X - sbStartX, 0))
					sbHandle.Left = sbHandle.Left - OldPos + pnlSidebar.Left
					If pnlGesture.IsInitialized Then pnlGesture.Left = pnlGesture.Left - OldPos + pnlSidebar.Left
					sbIsVisible = (pnlSidebar.Left <> -pnlSidebar.Width)
				Else
					OldPos = pnlContent.Left
					pnlContent.Left = Max(0, Min(pnlContent.Left + X - sbStartX, pnlSidebar.Width))
					If sbAnimType = 1 Then pnlSidebar.Left = pnlContent.Left - pnlSidebar.Width
					sbHandle.Left = sbHandle.Left - OldPos + pnlContent.Left
					If pnlGesture.IsInitialized Then pnlGesture.Left = pnlGesture.Left - OldPos + pnlContent.Left
					sbIsVisible = (pnlContent.Left <> 0)
				End If
			Case 1 'RIGHT
				If sbAnimType = 2 Then
					OldPos = pnlSidebar.Left
					pnlSidebar.Left = Max(pnlContent.Width - pnlSidebar.Width, Min(pnlSidebar.Left + X - sbStartX, pnlContent.Width))
					sbHandle.Left = sbHandle.Left - OldPos + pnlSidebar.Left
					If pnlGesture.IsInitialized Then pnlGesture.Left = pnlGesture.Left - OldPos + pnlSidebar.Left
					sbIsVisible = (pnlSidebar.Left <> pnlContent.Width)
				Else
					OldPos = pnlContent.Left
					pnlContent.Left = Max(-pnlSidebar.Width, Min(pnlContent.Left + X - sbStartX, 0))
					If sbAnimType = 1 Then pnlSidebar.Left = pnlContent.Left + pnlContent.Width
					sbHandle.Left = sbHandle.Left - OldPos + pnlContent.Left
					If pnlGesture.IsInitialized Then pnlGesture.Left = pnlGesture.Left - OldPos + pnlContent.Left
					sbIsVisible = (pnlContent.Left <> 0)
				End If
			Case 2 'TOP
				If sbAnimType = 2 Then
					OldPos = pnlSidebar.Top
					pnlSidebar.Top = Max(-pnlSidebar.Height, Min(pnlSidebar.Top + Y - sbStartY, 0))
					sbHandle.Top = sbHandle.Top - OldPos + pnlSidebar.Top
					If pnlGesture.IsInitialized Then pnlGesture.Top = pnlGesture.Top - OldPos + pnlSidebar.Top
					sbIsVisible = (pnlSidebar.Top <> -pnlSidebar.Height)
				Else
					OldPos = pnlContent.Top
					pnlContent.Top = Max(0, Min(pnlContent.Top + Y - sbStartY, pnlSidebar.Height))
					If sbAnimType = 1 Then pnlSidebar.Top = pnlContent.Top - pnlSidebar.Height
					sbHandle.Top = sbHandle.Top - OldPos + pnlContent.Top
					If pnlGesture.IsInitialized Then pnlGesture.Top = pnlGesture.Top - OldPos + pnlContent.Top
					sbIsVisible = (pnlContent.Top <> 0)
				End If
			Case 3 'BOTTOM
				If sbAnimType = 2 Then
					OldPos = pnlSidebar.Top
					pnlSidebar.Top = Max(pnlContent.Height - pnlSidebar.Height, Min(pnlSidebar.Top + Y - sbStartY, pnlContent.Height))
					sbHandle.Top = sbHandle.Top - OldPos + pnlSidebar.Top
					If pnlGesture.IsInitialized Then pnlGesture.Top = pnlGesture.Top - OldPos + pnlSidebar.Top
					sbIsVisible = (pnlSidebar.Top <> pnlContent.Height)
				Else
					OldPos = pnlContent.Top
					pnlContent.Top = Max(-pnlSidebar.Height, Min(pnlContent.Top + Y - sbStartY, 0))
					If sbAnimType = 1 Then pnlSidebar.Top = pnlContent.Top + pnlContent.Height
					sbHandle.Top = sbHandle.Top - OldPos + pnlContent.Top
					If pnlGesture.IsInitialized Then pnlGesture.Top = pnlGesture.Top - OldPos + pnlContent.Top
					sbIsVisible = (pnlContent.Top <> 0)
				End If
		End Select
	End If
	Return True
End Sub

'Enables the swipe gesture to open/close the sidebar
'<B>GestureAreaSize</B> = size of the gesture area
'The gesture area is an area above the frontier between the sidebar and the content panel where the gesture is expected.
'This area consumes all Touch events.
Public Sub EnableSwipeGesture(Enabled As Boolean, GestureAreaSize As Int)
	If Not(Enabled) Then
		If pnlGesture.IsInitialized Then pnlGesture.RemoveView
		pnlGesture = Null
		Return
	End If

	If pnlGesture.IsInitialized Then
		Select sbPosition
			Case 0 'LEFT
				pnlGesture.SetLayout(pnlContent.Left - (GestureAreaSize / 2), 0, GestureAreaSize, getParentHeight)
			Case 1 'RIGHT
				pnlGesture.SetLayout(pnlContent.Left + pnlContent.Width - (GestureAreaSize / 2), 0, GestureAreaSize, getParentHeight)
			Case 2 'TOP
				pnlGesture.SetLayout(0, pnlContent.Top - (GestureAreaSize / 2), getParentWidth, GestureAreaSize)
			Case 3 'BOTTOM
				pnlGesture.SetLayout(0, pnlContent.Top + pnlContent.Height - (GestureAreaSize / 2), getParentWidth, GestureAreaSize)
		End Select
	Else
		pnlGesture.Initialize("")
		Select sbPosition
			Case 0 'LEFT
				sbParent.AddView(pnlGesture, pnlContent.Left - (GestureAreaSize / 2), 0, GestureAreaSize, getParentHeight)
			Case 1 'RIGHT
				sbParent.AddView(pnlGesture, pnlContent.Left + pnlContent.Width - (GestureAreaSize / 2), 0, GestureAreaSize, getParentHeight)
			Case 2 'TOP
				sbParent.AddView(pnlGesture, 0, pnlContent.Top - (GestureAreaSize / 2), getParentWidth, GestureAreaSize)
			Case 3 'BOTTOM
				sbParent.AddView(pnlGesture, 0, pnlContent.Top + pnlContent.Height - (GestureAreaSize / 2), getParentWidth, GestureAreaSize)
		End Select

		Dim r As Reflector
		r.Target = pnlGesture
		r.SetOnTouchListener("pnlGesture_Touch")
	End If

pnlGesture.Color = Colors.ARGB(128, 255, 128, 128)

	pnlGesture.Enabled = True
End Sub

Private Sub pnlGesture_Touch(ViewTag As Object, Action As Int, X As Float, Y As Float, MotionEvent As Object) As Boolean
	If Not(pnlGesture.Enabled) Then Return True

	Dim r As Reflector
	If Action > 0 Then
		r.Target = sbParent
		r.RunMethod2("requestDisallowInterceptTouchEvent", True, "java.lang.boolean")
	End If

	r.Target = sbVelocityTracker
	If Action = 0 Then		'DOWN
		If sbVelocityTracker <> Null Then r.RunMethod("recycle")
		sbVelocityTracker = r.RunStaticMethod("android.view.VelocityTracker", "obtain", Null, Null)
		r.Target = sbVelocityTracker
		r.RunMethod4("addMovement", Array As Object(MotionEvent), Array As String("android.view.MotionEvent"))
	Else If Action = 2 Then	'MOVE
		r.RunMethod4("addMovement", Array As Object(MotionEvent), Array As String("android.view.MotionEvent"))
	Else If Action = 1 Then	'UP
		r.RunMethod2("computeCurrentVelocity", 1, "java.lang.int")
		Dim Velocity As Float
		Select sbPosition
			Case 0	'LEFT
				Velocity = r.RunMethod("getXVelocity") / Density * 1.5
				If IsSidebarVisible Then
					If Velocity < -sbMinVelocity Then
						AnimateSidebar(True, sbCloseDuration / Abs(Velocity))
					End If
				Else If Velocity > sbMinVelocity Then
					AnimateSidebar(False, sbOpenDuration / Velocity)
				End If
			Case 1	'RIGHT
				Velocity = r.RunMethod("getXVelocity") / Density * 1.5
				If IsSidebarVisible Then
					If Velocity > sbMinVelocity Then
						AnimateSidebar(True, sbCloseDuration / Velocity)
					End If
				Else If Velocity < -sbMinVelocity Then
					AnimateSidebar(False, sbOpenDuration / Abs(Velocity))
				End If
			Case 2	'TOP
				Velocity = r.RunMethod("getYVelocity") / Density * 1.5
				If IsSidebarVisible Then
					If Velocity < -sbMinVelocity Then
						AnimateSidebar(True, sbCloseDuration / Abs(Velocity))
					End If
				Else If Velocity > sbMinVelocity Then
					AnimateSidebar(False, sbOpenDuration / Velocity)
				End If
			Case 3	'BOTTOM
				Velocity = r.RunMethod("getYVelocity") / Density * 1.5
				If IsSidebarVisible Then
					If Velocity > sbMinVelocity Then
						AnimateSidebar(True, sbCloseDuration / Velocity)
					End If
				Else If Velocity < -sbMinVelocity Then
					AnimateSidebar(False, sbOpenDuration / Abs(Velocity))
				End If
		End Select
		r.RunMethod("recycle")
		sbVelocityTracker = Null
	End If
	Return True
End Sub
